﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Runtime.CompilerServices
Imports WpfNotes.Helper

Public Class ShowCategoriesWindow
    Implements INotifyPropertyChanged

    Private _ctx As Context.NotesContext


    Private Sub ShowCategoriesWindow_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded
        _ctx = New Context.NotesContext
        CategoryList = _ctx.Categories
        SelectedCategory = CategoryList.FirstOrDefault

        DataContext = Me
    End Sub

    Private Sub ShowCategoriesWindow_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        _ctx.Dispose()
    End Sub





    Private _categoryList As ObservableCollection(Of Model.Category)
    Public Property CategoryList() As ObservableCollection(Of Model.Category)
        Get
            Return _categoryList
        End Get
        Set(ByVal value As ObservableCollection(Of Model.Category))
            _categoryList = value
            RaisePropertyChanged()
        End Set
    End Property

    Private _selectedCategory As Model.Category

    Public Property SelectedCategory() As Model.Category
        Get
            Return _selectedCategory
        End Get
        Set(ByVal value As Model.Category)
            _selectedCategory = value
            RaisePropertyChanged()
        End Set
    End Property


    Private _addNewCategoryCommand As ICommand

    Public Property AddNewCategoryCommand() As ICommand
        Get
            If _addNewCategoryCommand Is Nothing Then _
                _addNewCategoryCommand = New RelayCommand(AddressOf AddNewCategoryCommand_Execute,
                                             AddressOf AddNewCategoryCommand_CanExecute)
            Return _addNewCategoryCommand
        End Get
        Set(ByVal value As ICommand)
            _addNewCategoryCommand = value
            RaisePropertyChanged()
        End Set
    End Property

    Private Function AddNewCategoryCommand_CanExecute(obj As Object) As Boolean
        Return True
    End Function

    Private Sub AddNewCategoryCommand_Execute(obj As Object)
        Dim dialog = New AddCategoryWindow With {
            .Owner = Me,
            .DataContext = New Model.Category
        }
        If dialog.ShowDialog Then
            CategoryList.Add(CType(dialog.DataContext, Model.Category))
        End If
    End Sub


Private _saveCommand As ICommand
        Public Property SaveCommand() As ICommand
            Get
                If _saveCommand Is Nothing Then _
                    _saveCommand = New RelayCommand(AddressOf SaveCommand_Execute,
                                                               AddressOf SaveCommand_CanExecute)
                Return _saveCommand
            End Get
            Set(ByVal value As ICommand)
                _saveCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function SaveCommand_CanExecute(obj As Object) As Boolean
            Return True
        End Function

    Private Sub SaveCommand_Execute(obj As Object)
        _ctx.SaveChanges()
        DialogResult = True
    End Sub

    Private _abortCommand As ICommand

    Public Property AbortCommand() As ICommand
        Get
            If _abortCommand Is Nothing Then _
                _abortCommand = New RelayCommand(AddressOf AbortCommand_Execute,
                                             AddressOf AbortCommand_CanExecute)
            Return _abortCommand
        End Get
        Set(ByVal value As ICommand)
            _abortCommand = value
            RaisePropertyChanged()
        End Set
    End Property

    Private Function AbortCommand_CanExecute(obj As Object) As Boolean
        Return True
    End Function

    Private Sub AbortCommand_Execute(obj As Object)
        DialogResult = False
    End Sub




    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Protected Overridable Sub RaisePropertyChanged(<CallerMemberName> Optional ByVal prop As String = "")
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(prop))
    End Sub


End Class
