﻿


Namespace Workspace
    Public Class MainWorkspace
        Inherits BaseClasses.ViewModelBase

        Friend ReadOnly Ctx As Context.NotesContext


        Public Sub New()
            If IsInDesignMode Then

            Else
                Ctx = New Context.NotesContext
                HeaderWs = New HeaderWorkspace(Me)
                NotesWs = New NotesWorkspace(Me)
                StatusWs = New StatusWorkspace
                AddHandler Ctx.DataFileWasChanged, AddressOf DataFileChanged
            End If
        End Sub

        Private Sub DataFileChanged()
            StatusWs.LastMessage = "Die Datendatei wurde von anderer Stelle bearbeitet. Sie sollten die Daten neu einlesen."
        End Sub


#Region "Workspaces"

        Private _headerWs As HeaderWorkspace
        Public Property HeaderWs() As HeaderWorkspace
            Get
                Return _headerWs
            End Get
            Set(ByVal value As HeaderWorkspace)
                _headerWs = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _notesWs As NotesWorkspace
        Public Property NotesWs() As NotesWorkspace
            Get
                Return _notesWs
            End Get
            Set(ByVal value As NotesWorkspace)
                _notesWs = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _statusWs As StatusWorkspace
        Public Property StatusWs() As StatusWorkspace
            Get
                Return _statusWs
            End Get
            Set(ByVal value As StatusWorkspace)
                _statusWs = value
                RaisePropertyChanged()
            End Set
        End Property

#End Region



        Private _isInEditMode As Boolean

        Public Property IsInEditMode() As Boolean
            Get
                Return _isInEditMode
            End Get
            Set(ByVal value As Boolean)
                If _isInEditMode Then
                    GoToReadOnlyMode()
                Else
                    If IsLockedForChanges Then
                        MessageBox.Show("Leider ist die Datei gerade gesperrt. Versuche es später nochmals.")
                        Exit Property
                    End If
                    If Not Ctx.IsDataUpToDate Then
                        MessageBox.Show("Die Daten wurden anscheinden von anderer Stelle bearbeitet. Bitte aktualisieren Sie zuerst und stellen Sie anschliessend auf den Editierungsmodus.")
                        Exit Property
                    End If
                    GoToEditMode()
                End If
                _isInEditMode = value
                RaisePropertyChanged()
            End Set
        End Property



        Public ReadOnly Property IsLockedForChanges() As Boolean
            Get
                Return Ctx Is Nothing OrElse Ctx.HasPessimisticLock
            End Get
        End Property


#Region "Hilfsmethoden"

        Private Sub GoToEditMode()
            Ctx.LockForChanges()
            RaisePropertyChanged(NameOf(IsLockedForChanges))
        End Sub

        Private Sub GoToReadOnlyMode()
            Ctx.UnlockForChanges()
            Ctx.SaveChanges()
            RaisePropertyChanged(NameOf(IsLockedForChanges))
        End Sub

#End Region



    End Class
End Namespace