﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports WpfNotes.Helper
Imports WpfNotes.Model

Namespace Workspace
    Public Class NotesWorkspace
        Inherits BaseClasses.ViewModelBase

        Public Sub New()
            If Not IsInDesignMode Then
                Throw New Exception("Dieser Konstruktor ist nur für den DesignTime-Support!")
            Else
                AviableCategories = New ObservableCollection(Of Category) From {
                    New Category() With {.Title = "Normale TODOs", .Desc = "Die Kategorie welche ganz normale Todos enthält"},
                    New Category() With {.Title = "Wichtige TODOs", .Desc = "Die Kategorie welche wichtige Erledigungen enthält"}
                }
                AviableNotes = New ObservableCollection(Of Note) From {
                    New Note() With {.Title = "Test Nr.1", .Content = "Content Nr. 1 mit Text", .CategorieId = AviableCategories.First.ID},
                    New Note() With {.Title = "Test Nr.2", .Content = "Content Nr. 2 mit Text", .CategorieId = AviableCategories.First.ID},
                    New Note() With {.Title = "Test Nr.3", .Content = "Content Nr. 3 mit Text", .CategorieId = AviableCategories.First.ID}
                }
                AviableNotesView = CollectionViewSource.GetDefaultView(AviableNotes)
            End If
        End Sub

        Public Sub New(mainWs As MainWorkspace)
            If mainWs Is Nothing Then Throw New ArgumentNullException(NameOf(mainWs))
            Me.MainWs = mainWs
            ReloadData()

        End Sub

        Friend Sub ReloadData()
            AviableCategories = New ObservableCollection(Of Category)
            AviableNotes = New ObservableCollection(Of Note)

            AviableCategories = MainWs.Ctx.Categories

            SelectedFilterCategory = AviableCategoriesFilter.First
            AviableNotes = MainWs.Ctx.Notes
            AviableNotesView = CollectionViewSource.GetDefaultView(AviableNotes)
            AviableNotesView.Filter = New Predicate(Of Object)(AddressOf NotesFilter)
        End Sub

        Public ReadOnly Property MainWs As MainWorkspace

        Private _aviableNotesView As ICollectionView

        Public Property AviableNotesView() As ICollectionView
            Get
                Return _aviableNotesView
            End Get
            Set(ByVal value As ICollectionView)
                _aviableNotesView = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _aviableCategories As ObservableCollection(Of Category)
        Public Property AviableCategories() As ObservableCollection(Of Category)
            Get
                Return _aviableCategories
            End Get
            Set
                _aviableCategories = Value
                RaisePropertyChanged()
                RaisePropertyChanged(NameOf(AviableCategoriesFilter))
            End Set
        End Property

        Private _aviableNotes As ObservableCollection(Of Note)

        Friend Property AviableNotes() As ObservableCollection(Of Note)
            Get
                Return _aviableNotes
            End Get
            Set
                _aviableNotes = Value
                RaisePropertyChanged()
            End Set
        End Property


#Region "Filter"

        Private Function NotesFilter(obj As Object) As Boolean
            Dim note = DirectCast(obj, Note)
            If SelectedFilterCategory IsNot Nothing AndAlso SelectedFilterCategory.ID <> Guid.Empty AndAlso
               Not DirectCast(obj, Note).CategorieId.ToString = SelectedFilterCategory.ID.ToString Then Return False
            If Not String.IsNullOrEmpty(FilterString) AndAlso Not note.Title.ToLower.Contains(FilterString.ToLower) Then Return False
            If FilterFinished AndAlso note.Finished Then Return False
            Return True
        End Function

        Private _aviableCategoriesFilter As ObservableCollection(Of Category)
        Public ReadOnly Property AviableCategoriesFilter() As ObservableCollection(Of Category)
            Get
                _aviableCategoriesFilter = New ObservableCollection(Of Category)(AviableCategories.ToList)
                _aviableCategoriesFilter.Insert(0, New Category() With {.Title = "ALLE", .Desc = "Ohne Kategoriefilter", .ID = Guid.Empty})
                Return _aviableCategoriesFilter
            End Get
        End Property


        Private _selectedFilterCategory As Category

        Public Property SelectedFilterCategory() As Category
            Get
                Return _selectedFilterCategory
            End Get
            Set
                _selectedFilterCategory = Value
                RaisePropertyChanged()
                If AviableNotesView IsNot Nothing Then
                    AviableNotesView.MoveCurrentToFirst()
                    AviableNotesView.Refresh()
                End If
            End Set
        End Property

        Private _filterString As String

        Public Property FilterString() As String
            Get
                Return _filterString
            End Get
            Set(ByVal value As String)
                _filterString = value
                RaisePropertyChanged()
                If AviableNotesView IsNot Nothing Then AviableNotesView.Refresh()
            End Set
        End Property

        Private _filterFinished As Boolean

        Public Property FilterFinished() As Boolean
            Get
                Return _filterFinished
            End Get
            Set(ByVal value As Boolean)
                _filterFinished = value
                RaisePropertyChanged()
                If AviableNotesView IsNot Nothing Then AviableNotesView.Refresh()
            End Set
        End Property


#End Region







#Region "Commands"

        Private _addNoteCommand As ICommand
        Public Property AddNoteCommand() As ICommand
            Get
                If _addNoteCommand Is Nothing Then _
                    _addNoteCommand = New RelayCommand(AddressOf AddNoteCommand_Execute,
                                                               AddressOf AddNoteCommand_CanExecute)
                Return _addNoteCommand
            End Get
            Set(ByVal value As ICommand)
                _addNoteCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function AddNoteCommand_CanExecute(obj As Object) As Boolean
            Return MainWs IsNot Nothing AndAlso MainWs.IsInEditMode
        End Function

        Private Sub AddNoteCommand_Execute(obj As Object)
            AviableNotes.Add(New Note With {.CategorieId = AviableCategories.First.ID})
            AviableNotesView.Refresh()
            AviableNotesView.MoveCurrentToLast()
        End Sub


        Private _removeNoteCommand As ICommand
        Public Property RemoveNoteCommand() As ICommand
            Get
                If _removeNoteCommand Is Nothing Then _
                    _removeNoteCommand = New RelayCommand(AddressOf RemoveNoteCommand_Execute,
                                                 AddressOf RemoveNoteCommand_CanExecute)
                Return _removeNoteCommand
            End Get
            Set(ByVal value As ICommand)
                _removeNoteCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function RemoveNoteCommand_CanExecute(obj As Object) As Boolean
            Return MainWs IsNot Nothing AndAlso MainWs.IsInEditMode AndAlso AviableNotesView.CurrentItem IsNot Nothing
        End Function

        Private Sub RemoveNoteCommand_Execute(obj As Object)
            Dim note As Note = CType(AviableNotesView.CurrentItem, Note)
            If MessageBox.Show("Wirklich löschen?", "Löschen?", MessageBoxButton.YesNo, MessageBoxImage.Question) = MessageBoxResult.Yes Then
                AviableNotes.Remove(note)
            End If
        End Sub


        Private _addAttachmentCommand As ICommand
        Public Property AddAttachmentCommand() As ICommand
            Get
                If _addAttachmentCommand Is Nothing Then _
                    _addAttachmentCommand = New RelayCommand(AddressOf AddAttachmentCommand_Execute,
                                                               AddressOf AddAttachmentCommand_CanExecute)
                Return _addAttachmentCommand
            End Get
            Set(ByVal value As ICommand)
                _addAttachmentCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function AddAttachmentCommand_CanExecute(obj As Object) As Boolean
            Return MainWs IsNot Nothing AndAlso MainWs.IsInEditMode
        End Function

        Private Sub AddAttachmentCommand_Execute(obj As Object)
            Dim addWindow As New AddEditAttachment
            Dim newFile As New Attachment
            addWindow.DataContext = newFile
            If addWindow.ShowDialog() Then
                DirectCast(AviableNotesView.CurrentItem, Note).Attachments.Add(newFile)
            End If

        End Sub

        Private _editAttachmentCommand As ICommand

        Public Property EditAttachmentCommand() As ICommand
            Get
                If _editAttachmentCommand Is Nothing Then _
                    _editAttachmentCommand = New RelayCommand(AddressOf EditAttachmentCommand_Execute,
                                                 AddressOf EditAttachmentCommand_CanExecute)
                Return _editAttachmentCommand
            End Get
            Set(ByVal value As ICommand)
                _editAttachmentCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function EditAttachmentCommand_CanExecute(obj As Object) As Boolean
            Return MainWs IsNot Nothing AndAlso obj IsNot Nothing AndAlso MainWs.IsInEditMode
        End Function

        Private Sub EditAttachmentCommand_Execute(obj As Object)
            Dim editWindow As New AddEditAttachment With {
                .DataContext = DirectCast(AviableNotesView.CurrentItem, Note).Attachments.Where(Function(a) a.ID.ToString = obj.ToString).FirstOrDefault
            }
            editWindow.ShowDialog()
        End Sub

        Private _removeAttachmentCommand As ICommand

        Public Property RemoveAttachmentCommand() As ICommand
            Get
                If _removeAttachmentCommand Is Nothing Then _
                    _removeAttachmentCommand = New RelayCommand(AddressOf RemoveAttachmentCommand_Execute,
                                                 AddressOf RemoveAttachmentCommand_CanExecute)
                Return _removeAttachmentCommand
            End Get
            Set(ByVal value As ICommand)
                _removeAttachmentCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function RemoveAttachmentCommand_CanExecute(obj As Object) As Boolean
            Return MainWs IsNot Nothing AndAlso obj IsNot Nothing AndAlso MainWs.IsInEditMode
        End Function

        Private Sub RemoveAttachmentCommand_Execute(obj As Object)
            If MessageBox.Show("Möchten Sie den gewählten Anhang wirklich entfernen", "Anhang löschen?", MessageBoxButton.YesNo) = MessageBoxResult.Yes Then
                Dim currentFile = DirectCast(AviableNotesView.CurrentItem, Note).Attachments.Where(Function(a) a.ID.ToString = obj.ToString).FirstOrDefault
                DirectCast(AviableNotesView.CurrentItem, Note).Attachments.Remove(currentFile)
            End If
        End Sub

        Private _openAttachmentCommand As ICommand

        Public Property OpenAttachmentCommand() As ICommand
            Get
                If _openAttachmentCommand Is Nothing Then _
                    _openAttachmentCommand = New RelayCommand(AddressOf OpenAttachmentCommand_Execute,
                                                 AddressOf OpenAttachmentCommand_CanExecute)
                Return _openAttachmentCommand
            End Get
            Set(ByVal value As ICommand)
                _openAttachmentCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function OpenAttachmentCommand_CanExecute(obj As Object) As Boolean
            Return MainWs IsNot Nothing AndAlso obj IsNot Nothing
        End Function

        Private Sub OpenAttachmentCommand_Execute(obj As Object)
            Process.Start(obj.ToString)
        End Sub

#End Region


    End Class
End Namespace